#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <SPI.h>
#include <MyConfig.h>
#include <MySensor.h>
//#include <MydebugLEDS.h>

const char* sensorData[] = {"TEMP","HUM","LIGHT","DIMMER","PRESSURE","FORECAST","RAIN","RAINRATE","WIND","GUST",
                            "DIRECTON","UV","WEIGHT","DISTANCE","IMPEDANCE","ARMED","TRIPPED","WATT","KWH","SCENE_ON",
                            "SCENE_OFF","HEATER","HEATER_SW","LIGHT_LEVEL","VAR1","VAR2","VAR3","VAR4","VAR5","UP",
                            "DOWN","STOP","IR_SEND","IR_RECEIVE","FLOW","VOLUME","LOCK_STATUS","DUST_LEVEL","VOLTAGE","CURRENT"};


typedef struct 
{
  int id;
  int child;
  int messType;
  int subType;
  int payloadType;
  const char* payload;
  const char* protocol;
} structIDs;
const int structs = 10;
structIDs IDs[structs];

LiquidCrystal_I2C lcd(0x38, 7,  POSITIVE);

//dLED debugLED1(4);
//dLED debugLED2(5);
//dLED debugLED3(6);
//dLED debugLED4(7);
MySensor gw;
bool LCDRefreshed = false;
bool notDone = true;
int nodeVisable;
int nodeOnDisplay = -1;
void setup(void)
{
  for (int i=0;i < structs;i++) {
    IDs[i].id = 255;
  }
  Serial.begin(115200);
  lcd.begin(20,4);
  gw.begin(incomingMessage, 0, true);
  gw.present(255,255);
  gw.openReadingPipe(1,BASE_RADIO_ID);
  gw.startListening();
  lcd.home ();
  lcd.setCursor ( 0, 0 );
  lcd.print("      MySensors");
  lcd.setCursor ( 0, 1 );
  lcd.print("  Node Scanner 1.2");
  lcd.setCursor ( 0, 2 );
  lcd.print("  Wait for a node.");
}

void loop(void)
{
  if (notDone) doOnce();
  else 
  gw.process();
}

void incomingMessage (const MyMessage &message) {
  // from transfer
  int node_id = message.sender;                //node-ID
  int child_sensor_id = message.sensor;        //s=
  int message_type = mGetCommand(message);     //c=
  int sub_type = message.type;                 //t=
  int payload_type = mGetPayloadType(message); //pt=
  const char* payload = message.getString();   //payload

  int strNr = searchID(node_id);
  if (strNr==255) strNr = newID();
  
  lcd.backlight();
  notDone = false;
  
  Serial.print(strNr);
  Serial.print("> from transfer < NodeID =");   Serial.print(node_id);
  Serial.print(" s=");                          Serial.print(child_sensor_id);
  Serial.print(" c=");                          Serial.print(message_type);
  Serial.print(" t=");                          Serial.print(sub_type);
  Serial.print(" pt=");                         Serial.print(payload_type);
  Serial.print(" payload=");                    Serial.println(payload);
  
  // put into struct
  if ((child_sensor_id == 255) && message_type == 0) IDs[strNr].protocol = payload;
  if ((child_sensor_id == 0) && (message_type != 3))
  switch (message_type) {
    case 0:
      IDs[strNr].id = node_id;
      if (child_sensor_id != 255) IDs[strNr].child = child_sensor_id;
      IDs[strNr].messType = message_type;
      IDs[strNr].payloadType = payload_type;
      
      IDs[strNr].subType = sub_type;
      LCDRefreshed = false;
    break;
    case 1:
      strNr = searchID(node_id);
      IDs[strNr].subType = sub_type;
      IDs[strNr].payloadType = payload_type;
      IDs[strNr].payload = payload;
      LCDRefreshed = false;
    break;
  }
  //from struct
  Serial.print(strNr);                    
  Serial.print("> from struct   < NodeID =");   Serial.print(IDs[strNr].id);
    Serial.print(" s=");                        Serial.print(IDs[strNr].child);
  Serial.print(" c=");                          Serial.print(IDs[strNr].messType);
  Serial.print(" t=");                          Serial.print(IDs[strNr].subType);
  Serial.print(" pt=");                         Serial.print(IDs[strNr].payloadType);
  Serial.print(" protocol=");                   Serial.print(IDs[strNr].protocol);
  Serial.print(" payload=");                    Serial.println(IDs[strNr].payload);

  if (!LCDRefreshed) showNodes(strNr);
}

void showNodes(int nodeNr) {
 // debugLED3.ON();
//  if ((nodeOnDisplay != nodeNr) && (IDs[nodeNr].child != 255)) {
    //debugLED4.ON();
    nodeOnDisplay = nodeNr;
    lcd.clear ();
    lcd.print(nodeOnDisplay);
    lcd.print(" Node ");
    lcd.print(IDs[nodeOnDisplay].id);
    lcd.print(".");
    lcd.print(IDs[nodeOnDisplay].child);
    lcd.print(" (");
    lcd.print(IDs[nodeOnDisplay].protocol);
    lcd.print(")");
    lcd.setCursor ( 2, 1 );
    lcd.print(sensorData[IDs[nodeOnDisplay].subType]);
    lcd.setCursor ( 2, 3 );
    lcd.print("Payload : ");
    lcd.print(IDs[nodeOnDisplay].payload);
    LCDRefreshed = true;
//  }
}

void doOnce() {
  int tmp = 0;
  while (notDone) {
    for (int i=0;i<4;i++) {
      lcd.setCursor ( i+4, 3 );
      lcd.print("    *    ");
      delay (200);
    }
    for (int i=4;i>0;i--) {
      lcd.setCursor ( i+4, 3 );
      lcd.print("    *    ");
      delay (200);
    }
    gw.process();
    tmp++;
    if (tmp == 5) lcd.noBacklight();
  }
}

int searchID(int tmpSender) {
  //debugLED1.ON();
  for (int i=0;i<structs;i++) {
    if (IDs[i].id == tmpSender) {
      Serial.print("ID search = ");
      Serial.println(i);
      return i;
    }
    if (i == structs) return 255;
  }
}

int newID() {
  //debugLED2.ON();
  for (int i=0;i<structs;i++) {
    if (IDs[i].id == 255) {
      Serial.print("ID new = ");
      Serial.println(i);
      return i;
    }
  }
}

